import os
import sys
sys.dont_write_bytecode = True
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import db
import timeSeries as TS
from datetime import datetime, date, timedelta
import time
from math import pi
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.models import DatetimeTickFormatter
from bokeh.palettes import all_palettes
from bokeh.models import HoverTool
from bokeh.embed import components
import itertools as itt


def removeNA(df, subset):
    df = df.dropna(subset=subset)
    return df

def indexTime(df, frm='%Y-%m-%d %H:%M:%S'):
    df['time'] = pd.to_datetime(df['time'], format=frm)
    df.index = df['time']
    del df['time']
    return df

def CruiseQuery(table, cruise):
    query = "SELECT [time], lat, lon FROM %s WHERE "
    query = query + "cruise='%s' "
    query = query % (table, cruise)
    return query

def getCruiseTrack(DB, source, cruise):
    df = None
    if DB:
        query = CruiseQuery(source, cruise)
        df = db.dbFetch(query)
    else:
        df = pd.read_csv(source)    
    return df

def resample(df, resampTau):
    if resampTau != '0':
        df = indexTime(df)
        df = df.resample(resampTau).mean()
        df.reset_index(level=0, inplace=True)
        df = removeNA(df, ['lat', 'lon'])        
    return df

def dumpCruiseShape(df, source, cruise, resampTau, fname):
    df = resample(df, resampTau)
    del df['time']  
    df['geometry'] = df.apply(lambda x: Point((float(x.lon), float(x.lat))), axis=1)
    df = gpd.GeoDataFrame(df, geometry='geometry')
    dirPath = 'shape/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)       
    df.to_file(dirPath + '%s.shp' % fname, driver='ESRI Shapefile')    
    return

def resampleToTimeResolution(resampTau):
    unit = resampTau[-1]
    sp = resampTau.split(unit)
    factor = sp[0]
    if len(factor) == 0:
        factor = 1
    else:
        factor = int(factor)
    if unit == 'T':
        dt = 1
    elif unit == 'H':
        dt = 60
    elif unit == 'D':
        dt = 24*60
    dt = factor * dt            
    return dt

def appendVar(cruiseTrack, t, y, yErr, variable, extV, extVV, extV2, extVV2):
    df = cruiseTrack
    df[variable] = y
    df[variable+'_std'] = yErr
    if extV != None:
        df[extV] = extVV
    if extV2 != None:
        df[extV2] = extVV2
    return df

def exportData(cruiseTrack, t, y, yErr, cruiseName, table, variable, margin, extV, extVV, extV2, extVV2):
    df = cruiseTrack
    df['margin'] = margin
    dirPath = 'data/'
    path = dirPath + 'Cruise_' + cruiseName + '.csv'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)    
    df.to_csv(path, index=False)    
    return


def plotAlongTrack(tables, variables, cruiseName, track, spMargin, extV, extVV, extV2, extVV2, exportDataFlag, marker='-', msize=30, clr='purple'):
    p = []
    fmt = '%Y-%m-%d %H:%M:%S'
    dt = resampleToTimeResolution(resampTau)         # time resolution (minutes)
    loadedTrack = pd.DataFrame(track)
    lw = 2
    w = 800
    h = 400
    TOOLS = 'pan,wheel_zoom,zoom_in,zoom_out,box_zoom, undo,redo,reset,tap,save,box_select,poly_select,lasso_select'
    for i in range(len(tables)):
        ts, ys, y_stds = [], np.array([]), np.array([])
        for j in range(len(track)):
            startDate = track.iloc[j]['time'].strftime(fmt)
            endDate = startDate
            lat1 = float(track.iloc[j]['lat']) - spMargin
            lat2 = float(track.iloc[j]['lat']) + spMargin
            lon1 = float(track.iloc[j]['lon']) - spMargin
            lon2 = float(track.iloc[j]['lon']) + spMargin           
            t, y, y_std = TS.timeSeries(tables[i], variables[i], startDate, endDate, lat1, lat2, lon1, lon2, extV[i], extVV[i], extV2[i], extVV2[i], fmt=fmt, dt=dt)
            ts.append(track.iloc[j]['time'])
            ys = np.append(ys, y[0])            
            y_stds = np.append(y_stds, y_std[0])
        loadedTrack = appendVar(loadedTrack, ts, ys, y_stds, variables[i], extV[i], extVV[i], extV2[i], extVV2[i])            
        p1 = figure(tools=TOOLS, toolbar_location="above", plot_width=w, plot_height=h)
        #p1.xaxis.axis_label = 'Time'
        p1.yaxis.axis_label = variables[i] + ' [' + db.getVar(tables[i], variables[i]).iloc[0]['Unit'] + ']'
        leg = 'Along Track (' + cruiseName + ') ' + variables[i]
        if extV[i] != None:
            leg = leg + '   ' + extV[i] + ': ' + ( '%d' % float(extVV[i]) ) 
            if tables[i].find('Pisces') != -1:
                leg = leg + ' ' + 'm'
        fill_alpha = 0.07        
        if tables[i].find('Pisces') != -1:
            fill_alpha = 0.3
        cr = p1.circle(ts, ys, fill_color="grey", hover_fill_color="firebrick", fill_alpha=fill_alpha, hover_alpha=0.3, line_color=None, hover_line_color="white", legend=leg, size=msize)
        p1.line(ts, ys, line_color=clr, line_width=lw, legend=leg)
        p1.add_tools(HoverTool(tooltips=None, renderers=[cr], mode='hline'))
        p1.xaxis.formatter=DatetimeTickFormatter(
                hours=["%d %B %Y %H:%M:%S"],
                days=["%d %B %Y %H:%M:%S"],
                months=["%d %B %Y %H:%M:%S"],
                years=["%d %B %Y %H:%M:%S"],
            )
        p1.xaxis.major_label_orientation = pi/4
        p.append(p1)

    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)    
    output_file(dirPath + fname + ".html", title="Along Track")
    show(column(p))
    if exportDataFlag:
        exportData(loadedTrack, ts, ys, y_stds, cruiseName, tables[i], variables[i], spMargin, extV[i], extVV[i], extV2[i], extVV2[i])    
    return loadedTrack


def mutualTrends(loadedTrack, tables, variables, extV, extVV, extV2, extVV2, msize=20, fname='MutualTrends'):
    if len(variables)<2:
        print('Error: at least 2 variables are needed to plot mutual trends.')
        return
    tablePairs = list(itt.combinations(tables, 2))
    variablePairs = list(itt.combinations(variables, 2))
    extVPairs = list(itt.combinations(extV, 2))
    extVVPairs = list(itt.combinations(extVV, 2))
    extV2Pairs = list(itt.combinations(extV2, 2))
    extVV2Pairs = list(itt.combinations(extVV2, 2))


    p = []
    lw = 2
    w = 500
    h = 500
    TOOLS = 'pan,wheel_zoom,zoom_in,zoom_out,box_zoom, undo,redo,reset,tap,save,box_select,poly_select,lasso_select'
    for i in range(len(tablePairs)):
        t1, y1, y_std1 = loadedTrack['time'], loadedTrack[variablePairs[i][0]], loadedTrack[variablePairs[i][0] + '_std']
        t2, y2, y_std2 = loadedTrack['time'], loadedTrack[variablePairs[i][1]], loadedTrack[variablePairs[i][1] + '_std']

        p1 = figure(tools=TOOLS, toolbar_location="above", plot_width=w, plot_height=h)
        p1.xaxis.axis_label = variablePairs[i][0] + ' [' + db.getVar(tablePairs[i][0], variablePairs[i][0]).iloc[0]['Unit'] + ']'
        p1.yaxis.axis_label = variablePairs[i][1] + ' [' + db.getVar(tablePairs[i][1], variablePairs[i][1]).iloc[0]['Unit'] + ']'
        leg = variablePairs[i][0] + ' / ' + variablePairs[i][1]
        if extVPairs[i][0] != None:
            leg = leg + '   ' + extVPairs[i][0] + ': ' + ( '%d' % float(extVVPairs[i][0]) ) 
            if tablePairs[i][0].find('Pisces') != -1:
                leg = leg + ' ' + 'm'
        if extVPairs[i][1] != None:
            leg = leg + '   ' + extVPairs[i][1] + ': ' + ( '%d' % float(extVVPairs[i][1]) ) 
            if tablePairs[i][1].find('Pisces') != -1:
                leg = leg + ' ' + 'm'
        fill_alpha = 0.6        
        #if tablePairs[i][0].find('Pisces') != -1 or tablePairs[i][1].find('Pisces') != -1:
        #    fill_alpha = 0.3
        cr = p1.circle(y1, y2, fill_color="grey", hover_fill_color="firebrick", fill_alpha=fill_alpha, hover_alpha=0.6, line_color=None, hover_line_color="white", legend=leg, size=msize)
        #p1.line(y1, y2, line_color=clr, line_width=lw, legend=leg)
        p1.add_tools(HoverTool(tooltips=None, renderers=[cr], mode='hline'))    
        p.append(p1)
    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        
    output_file(dirPath + fname + ".html", title="Mutual Trends")
    show(column(p))
    return


DB = bool(int(sys.argv[1]))
command = int(sys.argv[2])
source = sys.argv[3]      
cruise = sys.argv[4]      
resampTau = sys.argv[5]
fname = sys.argv[6] 
df = getCruiseTrack(DB, source, cruise)
df = resample(df, resampTau)
if command == 1:        ## generates cruise track shapefile
    import geopandas as gpd
    from shapely.geometry import Point    
    dumpCruiseShape(df, source, cruise, resampTau, fname)
elif command == 2:      ## generates along track plot
    exportDataFlag = bool(int(sys.argv[7]))
    spMargin = float(sys.argv[8])         #spatial margin 
    tables = sys.argv[9].split(',')
    variables = sys.argv[10].split(',')   
    extV = sys.argv[11].split(',')       #extra condition: var_name
    extVV = sys.argv[12].split(',')       #extra condition: var_val
    extV2 = sys.argv[13].split(',')       #extra condition: var_name
    extVV2 = sys.argv[14].split(',')       #extra condition: var_val  
    for i in range(len(tables)):
        if extV[i].find('ignore') != -1:
            extV[i]=None
        if extVV[i].find('ignore') != -1:
            extVV[i]=None
        if extV2[i].find('ignore') != -1:
            extV2[i]=None
        if extVV2[i].find('ignore') != -1:
            extVV2[i]=None
    loadedTrack = plotAlongTrack(tables, variables, cruise, df, spMargin, extV, extVV, extV2, extVV2, exportDataFlag, marker='-', msize=30, clr='darkturquoise')
    
    mutualTrends(loadedTrack, tables, variables, extV, extVV, extV2, extVV2, cruise)



import sys
import os
import numpy as np
import pandas as pd
from datetime import datetime, timedelta
import db
import timeSeries as TS
import matplotlib.pyplot as plt
from math import pi
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.models import DatetimeTickFormatter
from bokeh.palettes import all_palettes
from bokeh.models import HoverTool
from bokeh.embed import components
from tqdm import tqdm
   


def prepareQuery(t, lat, lon):
    table = 'tblAltimetry_REP'
    u, v = 'ugos', 'vgos' 
    Xres, Yres = 0.25/2, 0.25/2
    args = (u, v, table, t, lat, Yres, lon, Xres, lat, lon)
    query = "SELECT [time], lat, lon, %s AS u, %s AS v FROM %s WHERE "
    query = query + "[time]='%s' AND "
    query = query + "ABS(lat-(%f))<=%f AND ABS(lon-(%f))<=%f "
    #query = query + "AND %s IS NOT NULL AND %s IS NOT NULL "
    query = query + "ORDER BY ABS(lat-(%f)), ABS(lon-(%f))"
    query = query % args
    return query


def weightedMean(df, lat, lon):
    u, v = 0, 0
    df['r'] = np.power(np.power(df['lat']-lat, 2) + np.power(df['lon']-lon, 2), 0.5)
    n = len(df)
    U, V, R = np.array(df['u']), np.array(df['v']), np.array(df['r'])  
    totalW = 0
    for i in range(n):
        if R[i] != 0:
            totalW = totalW + 1/R[i]
        else:
            return U[i], V[i]          
    for i in range(n):        
        u = u + U[i]/R[i]    
        v = v + V[i]/R[i]
    u = u/totalW    
    v = v/totalW  
    return u, v


def interpolateUV(t, lat, lon):
    query = prepareQuery(t, lat, lon)
    df = db.dbFetch(query)        
    df = pd.DataFrame.from_records(df, columns=['time', 'lat', 'lon', 'u', 'v'])
    n = len(df)
    u, v = 0, 0
    if n == 1:
        u, v = np.array(df['u'])[0], np.array(df['v'])[0]
    else:
        u, v = weightedMean(df, lat, lon)            
    if u == None:
        u = 0
    if v == None:
        v = 0
    return u, v


def propagate(direction, startDate, endDate, lat, lon, fmt, dt):
    def cond(direction, startDate, endDate, t):
        if direction==1:
            res = t<=endDate
        elif direction==-1:
            res = t>=startDate
        return res

    dx = 111180 * np.cos(lat*np.pi/180)
    dy = 111180

    ts, lats, lons = np.array([]), np.array([]), np.array([])
    startDate = datetime.strptime(startDate, fmt)
    endDate = datetime.strptime(endDate, fmt)
    if endDate < startDate:
        swap = startDate
        startDate = endDate
        endDate = swap

    if direction==1:
        t = startDate
    elif direction==-1:
        t = endDate

    ts = np.append(ts, t)
    lats = np.append(lats, lat)
    lons = np.append(lons, lon) 
    
    while cond(direction, startDate, endDate, t):  
        ##### interpolate flowfield at particle's location
        u, v = interpolateUV(t.strftime(fmt), lat, lon)
        ##### advection
        lon = lon + direction*dt*u / dx
        lat = lat + direction*dt*v / dy
        t = t + timedelta(seconds=direction*dt)
        #### update trajectory
        ts = np.append(ts, t)
        lats = np.append(lats, lat)
        lons = np.append(lons, lon)     
    return ts, lats, lons


def dumpTrackShape(lats, lons, fname):
    df = pd.DataFrame()
    df['lat'] = lats
    df['lon'] = lons
    df['geometry'] = df.apply(lambda x: Point((float(x.lon), float(x.lat))), axis=1)
    df = gpd.GeoDataFrame(df, geometry='geometry')
    dirPath = 'shape/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)       
    df.to_file(dirPath + '%s.shp' % fname, driver='ESRI Shapefile')    
    return

def appendVar(track, t, y, yErr, variable, extV, extVV, extV2, extVV2):
    df = track
    df[variable] = y
    df[variable+'_std'] = yErr
    if extV != None:
        df[extV] = extVV
    if extV2 != None:
        df[extV2] = extVV2
    return df


def exportData(cruiseTrack, t, y, yErr, table, variable, margin, extV, extVV, extV2, extVV2):
    df = cruiseTrack
    df['margin'] = margin
    dirPath = 'data/'
    path = dirPath + 'Tracer.csv'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)    
    df.to_csv(path, index=False)    
    return


def plotAlongTrack(dt, fmt, tables, variables, track, spMargin, extV, extVV, extV2, extVV2, exportDataFlag, fname, marker='-', msize=30, clr='purple'):
    p = []
    #fmt = '%Y-%m-%d %H:%M:%S'
    dt = dt/60         # time resolution (minutes)
    loadedTrack = pd.DataFrame(track)
    lw = 2
    w = 800
    h = 400
    TOOLS = 'pan,wheel_zoom,zoom_in,zoom_out,box_zoom, undo,redo,reset,tap,save,box_select,poly_select,lasso_select'
    for i in tqdm(range(len(tables))):
        ts, ys, y_stds = [], np.array([]), np.array([])
        for j in tqdm(range(len(track))):
            startDate = track.iloc[j]['time'].strftime(fmt)
            endDate = startDate
            lat1 = float(track.iloc[j]['lat']) - spMargin
            lat2 = float(track.iloc[j]['lat']) + spMargin
            lon1 = float(track.iloc[j]['lon']) - spMargin
            lon2 = float(track.iloc[j]['lon']) + spMargin           
            t, y, y_std = TS.timeSeries(tables[i], variables[i], startDate, endDate, lat1, lat2, lon1, lon2, extV[i], extVV[i], extV2[i], extVV2[i], fmt=fmt, dt=dt)
            ts.append(track.iloc[j]['time'])
            ys = np.append(ys, y[0])            
            y_stds = np.append(y_stds, y_std[0])
        loadedTrack = appendVar(loadedTrack, ts, ys, y_stds, variables[i], extV[i], extVV[i], extV2[i], extVV2[i])            
        p1 = figure(tools=TOOLS, toolbar_location="above", plot_width=w, plot_height=h)
        #p1.xaxis.axis_label = 'Time'
        p1.yaxis.axis_label = variables[i] + ' [' + db.getVar(tables[i], variables[i]).iloc[0]['Unit'] + ']'
        leg = 'Along Tracer Track ' + variables[i]
        if extV[i] != None:
            leg = leg + '   ' + extV[i] + ': ' + ( '%d' % float(extVV[i]) ) 
            if tables[i].find('Pisces') != -1:
                leg = leg + ' ' + 'm'
        fill_alpha = 0.07        
        if tables[i].find('Pisces') != -1:
            fill_alpha = 0.3
        cr = p1.circle(ts, ys, fill_color="grey", hover_fill_color="firebrick", fill_alpha=fill_alpha, hover_alpha=0.3, line_color=None, hover_line_color="white", legend=leg, size=msize)
        p1.line(ts, ys, line_color=clr, line_width=lw, legend=leg)
        p1.add_tools(HoverTool(tooltips=None, renderers=[cr], mode='hline'))
        p1.xaxis.formatter=DatetimeTickFormatter(
                hours=["%d %B %Y"],
                days=["%d %B %Y"],
                months=["%d %B %Y"],
                years=["%d %B %Y"],
            )
        p1.xaxis.major_label_orientation = pi/4
        p.append(p1)

    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)    
    output_file(dirPath + fname + ".html", title="Lagrangian")
    show(column(p))
    if exportDataFlag:
        exportData(loadedTrack, ts, ys, y_stds, tables[i], variables[i], spMargin, extV[i], extVV[i], extV2[i], extVV2[i])    
    print('')
    return













fmt='%Y-%m-%d'

dt = int(sys.argv[1])           # time step (day seconds)
direction = int(sys.argv[2])    # integration direction (forward/backward in time)
startDate = sys.argv[3]
endDate = sys.argv[4]
lat0 = float(sys.argv[5])
lon0 = float(sys.argv[6])
shapeFlag = bool(int(sys.argv[7]))
colocateFlag = bool(int(sys.argv[8]))

if shapeFlag or colocateFlag:
    ts, lats, lons = propagate(direction, startDate, endDate, lat0, lon0, fmt, dt)

if shapeFlag:                       # make shapefile for the tracer's trajectory
    import geopandas as gpd
    from shapely.geometry import Point 
    ### shapefile params
    shapeFname = sys.argv[9]
    ##########################
    dumpTrackShape(lats, lons, shapeFname)

if colocateFlag:                    # colocate the tracer's trajectory on varialble fields
    #### colocalization params
    exportDataFlag = bool(int(sys.argv[10]))
    spMargin = float(sys.argv[11])         #spatial margin 
    tables = sys.argv[12].split(',')
    variables = sys.argv[13].split(',')   
    extV = sys.argv[14].split(',')        #extra condition: var_name
    extVV = sys.argv[15].split(',')       #extra condition: var_val
    extV2 = sys.argv[16].split(',')       #extra condition: var_name
    extVV2 = sys.argv[17].split(',')      #extra condition: var_val  
    fname = sys.argv[18]
    for i in range(len(tables)):
        if extV[i].find('ignore') != -1:
            extV[i]=None
        if extVV[i].find('ignore') != -1:
            extVV[i]=None
        if extV2[i].find('ignore') != -1:
            extV2[i]=None
        if extVV2[i].find('ignore') != -1:
            extVV2[i]=None
    #############################

    df = pd.DataFrame()
    df['time'] = ts
    df['lat'] = lats
    df['lon'] = lons
    plotAlongTrack(dt, fmt, tables, variables, df, spMargin, extV, extVV, extV2, extVV2, exportDataFlag, fname, marker='-', msize=30, clr='darkturquoise')











import sys
sys.dont_write_bytecode = True
import os
import db
import pandas as pd
from datetime import datetime
from datetime import timedelta  
import numpy as np
from tqdm import tqdm

def getPiscesDepthLevels():
    query = "SELECT * FROM tblPisces_Depth"
    depthLevs = db.dbFetch(query) 
    return depthLevs


def sourceQuery(srcTable):
    query  = 'SELECT * FROM %s '
    query = query % (srcTable)
    return query


def getSrcDF(DB, source):
    srcDF = None
    if DB:
        query = sourceQuery(source) 
        srcDF = db.dbFetch(query)   
    else:  
        filename, fileExtension = os.path.splitext(source)  
        if fileExtension == '.xlsx':
            srcDF = pd.read_excel(open(source,'rb'), sheetname='data')
        elif fileExtension == '.csv':    
            srcDF = pd.read_csv(source)
        srcDF['time'] = pd.to_datetime(srcDF['time'])
    return srcDF


def matchQuery(table, field, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2):
    args = (field, table, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2)
    query = "SELECT %s FROM %s WHERE "
    query = query + "[time]>='%s' AND [time]<='%s' AND "
    query = query + "lat>=%f AND lat<=%f AND "
    query = query + "lon>=%f AND lon<=%f AND "
    query = query + "depth>=%f AND depth<=%f "
    query = query % args
    return query


def localize(table, field, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2):
    query = matchQuery(table, field, startDate, endDate, lat1, lat2, lon1, lon2, depth1, depth2)
    df = db.dbFetch(query)        
    return df






def matchSource():
    target = []
    for i in range(len(tables)):
        target.append((tables[i], variables[i]))

    srcDF = getSrcDF(DB, source)
    depthLevs = getPiscesDepthLevels()
    srcDF['dt1'], srcDF['dt2'], srcDF['lat1'], srcDF['lat2'], srcDF['lon1'], srcDF['lon2'], srcDF['depth1'], srcDF['depth2'] = '', '', '', '', '', '', '', ''
    for tar in tqdm(target):
        srcDF[tar[1]] = ''
        srcDF[tar[1]+'_std'] = ''
        for i in tqdm(range(len(srcDF))):
            dt = srcDF.time[i]   
            dt1 = dt - timedelta(days=tMargin) 
            dt2 = dt + timedelta(days=tMargin) 
            lat = srcDF.lat[i]
            lon = srcDF.lon[i]
            depth = srcDF.depth[i]
            ind = np.abs(depthLevs['depth_level']-depth).idxmin()# argmin()
            depth = depthLevs['depth_level'][ind]
            lat1, lat2 = lat - latMargin, lat + latMargin
            lon1, lon2 = lon - lonMargin, lon + lonMargin
            depth1, depth2 = depth - 1, depth + 1
            data = localize(tar[0], tar[1], str(dt1), str(dt2), lat1, lat2, lon1, lon2, depth1, depth2)
            srcDF.loc[i, 'dt1'], srcDF.loc[i, 'dt2'], srcDF.loc[i, 'lat1'], srcDF.loc[i, 'lat2'], srcDF.loc[i, 'lon1'], srcDF.loc[i, 'lon2'], srcDF.loc[i, 'depth1'], srcDF.loc[i, 'depth2'] = dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2
            srcDF.loc[i,tar[1]] = np.mean(data[tar[1]])
            srcDF.loc[i, tar[1]+'_std'] = np.std(data[tar[1]])

    dirPath = os.path.dirname(exportPath)
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        
    srcDF.to_csv(exportPath, index=False)
    print('')
    return





DB = bool(int(sys.argv[1]))
source = sys.argv[2]      
latMargin = float(sys.argv[3])         #meridional margin 
lonMargin = float(sys.argv[3])         #zonal margin 
####depMargin = float(sys.argv[3])         #depth margin 
tMargin = float(sys.argv[4])         #temporal margin 
tables = sys.argv[5].split(',')
variables = sys.argv[6].split(',')   
exportPath = sys.argv[7] 

matchSource()



#! /usr/bin/env python

"""
usage:      
    opedia <command> [options]
    opedia (-h | --help | --version)


options:
   --tab <datasets>                      Query parameter: comma-separated data set names (tables) [default: tblSST_AVHRR_OI_NRT]    
   --var <variables>                     Query parameter: comma-separated variable names (fields) [default: sst]     
   --d1 <startDate>                      Query parameter: start date [default: 2016-04-30]  
   --d2 <endDate>                        Query parameter: end date [default: 2016-04-30]
   --h1 <startHour>                      Query parameter: start hour [default: 0]  
   --h2 <endHour>                        Query parameter: end hour [default: 0]
   --lat1 <startLat>                     Query parameter: start latitude [default: 20]
   --lat2 <endLat>                       Query parameter: end latitude [default: 45]
   --lon1 <startLon>                     Query parameter: start longitude [default: -180]
   --lon2 <endLon>                       Query parameter: end longitude [default: -126]
   --extV1 <extraV1>                     Query parameter: comma-separated extra query parameter names [default: ignore]   
   --extVV1 <extraVV1>                   Query parameter: comma-separated extra query parameter values [default: ignore]   
   --extV2 <extraV2>                     Query parameter: comma-separated extra query parameter names [default: ignore]   
   --extVV2 <extraVV2>                   Query parameter: comma-separated extra query parameter values [default: ignore]   
   --dep1 <startDepth>                   Query parameter: start depth [default: 0]
   --dep2 <endDepth>                     Query parameter: end depth [default: 0]
   -e <eflag>, --export <eflag>          Export data flag [default: 0]
   --eFile <dataFile>                    Export data file name [default: ./data/data.csv]   
   --shape <shapeFlag>                   Generate shape file [default: 1]
   --shpFile <shapeFile>                 Shape filename [default: shape]
   --match <matchFlag>                   Match/Colocate with other data sets [default: 0]
   --bkg <bkgFlag>                       Background comparison flag [default: 0]
   --pd <plotDir>                        Plot directory [default: ./embed]   
   --pname <plotFile>                    Plot file name [default: plot]   
   --sds <sourceDataset>                 Source data set(table name) to be matched with the matching data sets            
   --mds <matchDataset>...               Matching data sets(table names) to be matched with the source data set 
   --cruiseName <crsName>                Cruise name
   --fCruise <fcflag>                    Field (vs virtual) cruise flag [default: 1] 
   --cruiseTrack <crsTrack>              Generate cruise track shape file [default: 1] 
   --cruiseTab <cruiseTable>             Cruise data set (table) name  
   --vcFile <virtualCruiseFile>          Path to virtual cruise file 
   --resampleRate <resampleTau>          Resample cruise data set (disabled by default) [default: 0]
   
   --dbMatch <dbMatchFlag>               Match with a data set (table) in database (0 if local file) [default: 1]     
   --sourceMatch <srcMatch>              Source data set to be matched with (table name if dbMatch, file name if local file)
   --sMatch <spatialMatch>               Spatial matching range in degrees [default: 0.3]     
   --tMatch <spatialMatch>               Temporal matching range in days [default: 1]                           

   --dtAdvect <timeStep>                 Advection time step (in seconds, 3600*24=86400) [default: 86400]
   --dirAdvect <direction>               Advection direction (forward/backward 1/-1) [default: 1]
   --latAdvect <initialLat>              Initial latitude of the passive tracer
   --lonAdvect <initialLon>              Initial longitude of the passive tracer

   --eddySource <srcEddy>                Eddy data set (table) name [default: tblChelton]
   --ftleSource <srcFTLE>                FTLE data set (table) name [default: tblLCS_REP]
   --ftleVar <ftleField>                 FTLE field name [default: ftle_bw_sla]
   --ftleValue <ftleCutoff>              FTLE high-pass filter threshold [default: 0.22]

   --name                                Your name             
   --lastname                            Your last name             
   -u <username>, --username <username>  Your username [default: ArmLab]
   -p <password>, --password <password>  Your password [default: ArmLab2018]
   --email <email>                       Your email address   
   --server                              Server IP address [default: 128.208.239.15]  
   --port                                Server port number [default: 1433]        
   --db                                  Database name [default: Opedia]   
   -a, --all                             Print all options and arguments                   
   -h, --help                            Show usage doc string and exit



commands:
   auth                                  User authentication (NOT IMPLEMENTED YET)    
   regional                              Retrieve regional subset of data sets 
   timeseries                            Retrieve timeseries subset of data sets 
   trend                                 Retrieve mutual trend between a pair of data sets 
   dist                                  Retrieve a generic subset of data sets 
   section                               Retrieve section (depth vs lat/lon) subset of data sets    
   depth                                 Retrieve depth profile     
   season                                Retrieve data set seasonality    
   cat                                   Retrieve opedia catalog    
   cruise                                Retrieve and match cruise track with other data sets
   match                                 Match/Colocalize a data set with other data sets    
   advect                                Advect a paccive tracer and match/colocate other data sets along track    
   eddy                                  Retrieve and match/colocate eddy cores with other data sets    
   ftle                                  Retrieve and match/colocate ftle fronts with other data sets    




Examples:
__________________________________________________________________________________________________________


REGIONAL MAP:
    opedia regional [options] [--tab=<datasets>] [--var=<variables>] [--d1=<startDate>] [--lat1=<startLat>] [--lat2=<endLat>] [--lon1=<startLon>] [--lon2=<endLon>] [--pname=<plotFile>] [--export=<eflag>] [--extV1=<extraV1>] [--extVV1=<extraVV1>]
=====================================================

TIMESERIES:
    opedia timeseries [options] [--tab=<datasets>] [--var=<variables>] [--d1=<startDate>] [--d2=<endDate>] [--lat1=<startLat>] [--lat2=<endLat>] [--lon1=<startLon>] [--lon2=<endLon>] [--pname=<plotFile>] [--export=<eflag>] [--extV1=<extraV1>] [--extVV1=<extraVV1>] [--extV2=<extraV2>] [--extVV2=<extraVV2>]
=====================================================

MUTUAL TREND:
    opedia trend [options] [--tab=<datasets>] [--var=<variables>] [--d1=<startDate>] [--d2=<endDate>] [--lat1=<startLat>] [--lat2=<endLat>] [--lon1=<startLon>] [--lon2=<endLon>] [--pname=<plotFile>] [--export=<eflag>] [--extV1=<extraV1>] [--extVV1=<extraVV1>] [--extV2=<extraV2>] [--extVV2=<extraVV2>]
=====================================================

GENERIC DISTRIBUTION:
    opedia dist [options] [--tab=<datasets>] [--var=<variables>] [--d1=<startDate>] [--d2=<endDate>] [--lat1=<startLat>] [--lat2=<endLat>] [--lon1=<startLon>] [--lon2=<endLon>] [--pname=<plotFile>] [--export=<eflag>] [--extV1=<extraV1>] [--extVV1=<extraVV1>] [--extV2=<extraV2>] [--extVV2=<extraVV2>]
=====================================================

SECTION:
    opedia section [options] [--tab=<datasets>] [--var=<variables>] [--d1=<startDate>] [--lat1=<startLat>] [--lat2=<endLat>] [--lon1=<startLon>] [--lon2=<endLon>] [--pname=<plotFile>] [--export=<eflag>] [--extVV1=<extraVV1>] [--extVV2=<extraVV2>]
=====================================================

DEPTH PROFILE:
    opedia depth [options] [--tab=<datasets>] [--var=<variables>] [--d1=<startDate>] [--lat1=<startLat>] [--lat2=<endLat>] [--lon1=<startLon>] [--lon2=<endLon>] [--pname=<plotFile>] [--export=<eflag>] [--extVV1=<extraVV1>] [--extVV2=<extraVV2>]
=====================================================

SEASON:
    opedia season [options] [--tab=<datasets>] [--var=<variables>] [--lat1=<startLat>] [--lat2=<endLat>] [--lon1=<startLon>] [--lon2=<endLon>] [--pname=<plotFile>] [--export=<eflag>] [--extV1=<extraV1>] [--extVV1=<extraVV1>] [--extV2=<extraV2>] [--extVV2=<extraVV2>]
=====================================================

CATALOG:
    opedia cat [options]
=====================================================

CRUISE:
    opedia cruise [options] [--fCruise=<fCruise>] [--cruiseTrack=<crsTrack>] [--cruiseName=<crsName>] [--resampleRate=<resampleTau>] [--pname=<plotFile>] [--export=<eflag>] [--sMatch=<spatialMatch>] [--tab=<datasets>] [--var=<variables>] [--extV1=<extraV1>] [--extVV1=<extraVV1>] [--extV2=<extraV2>] [--extVV2=<extraVV2>]
=====================================================

MATCH:
    opedia match [options] [--dbMatch=<dbMatchFlag> ] [--sourceMatch=<srcMatch>] [--sMatch=<spatialMatch> ] [--tMatch=<spatialMatch> ] [--tab=<datasets>] [--var=<variables>] [--eFile=<dataFile>]
=====================================================

ADVECT:
    opedia advect [options] [--dtAdvect=<timeStep>] [--dirAdvect=<direction> ] [--d1=<startDate>] [--d2=<endDate>] [--latAdvect=<initialLat>] [--lonAdvect=<initialLon>] [--shape=<shapeFlag>] [--match=<matchFlag>] [--shpFile=<shapeFile>] [--export=<eflag>] [--sMatch=<spatialMatch>] [--tab=<datasets>] [--var=<variables>] [--extV1=<extraV1>] [--extVV1=<extraVV1>] [--extV2=<extraV2>] [--extVV2=<extraVV2>] [--pname=<plotFile>]
=====================================================

EDDY:
    opedia eddy [options] [--eddySource=<srcEddy>] [--d1=<startDate>] [--d2=<endDate>] [--lat1=<startLat>] [--lat2=<endLat>] [--lon1=<startLon>] [--lon2=<endLon>] [--shape=<shapeFlag>] [--match=<matchFlag>] [--shpFile=<shapeFile>] [--export=<eflag>] [--sMatch=<spatialMatch>] [--tab=<datasets>] [--var=<variables>] [--extV1=<extraV1>] [--extVV1=<extraVV1>] [--extV2=<extraV2>] [--extVV2=<extraVV2>] [--pname=<plotFile>]
=====================================================

FTLE:
    opedia ftle [options] [--ftleSource=<srcFTLE>] [--ftleVar=<ftleField>] [--ftleValue=<ftleCutoff>] [--bkg=<bkgFlag>] [--d1=<startDate>] [--d2=<endDate>] [--lat1=<startLat>] [--lat2=<endLat>] [--lon1=<startLon>] [--lon2=<endLon>] [--shape=<shapeFlag>] [--match=<matchFlag>] [--shpFile=<shapeFile>] [--export=<eflag>] [--sMatch=<spatialMatch>] [--tab=<datasets>] [--var=<variables>] [--extV1=<extraV1>] [--extVV1=<extraVV1>] [--extV2=<extraV2>] [--extVV2=<extraVV2>] [--pname=<plotFile>]
=====================================================

"""




from docopt import docopt
from subprocess import call
import os
import json


def run(cmd, subProc=True):
    if args['--all']:
        print(json.dumps(args, indent=5))
    if subProc:
        ## subprocess call
        exit(call(cmd))
    else:  
        ## os 
        os.system(' '.join(cmd))


if __name__ == '__main__':
    args = docopt(__doc__, version='opedia 0.1.34', options_first=False)

    if args['<command>'] == 'regional':
        elems = [args['--tab'], args['--var'], args['--d1'], args['--lat1'], args['--lat2'], args['--lon1'], args['--lon2'], args['--pname'], args['--export'], args['--extV1'], args['--extVV1'] ]
        run(['python', 'plotRegional.py'] + elems)

    elif args['<command>'] == 'timeseries':
        elems = [args['--tab'], args['--var'], args['--d1'], args['--d2'], args['--lat1'], args['--lat2'], args['--lon1'], args['--lon2'], args['--pname'], args['--export'], args['--extV1'], args['--extVV1'], args['--extV2'], args['--extVV2'] ]
        run(['python', 'plotTS.py'] + elems)

    elif args['<command>'] == 'trend':
        elems = [args['--tab'], args['--var'], args['--d1'], args['--d2'], args['--lat1'], args['--lat2'], args['--lon1'], args['--lon2'], args['--pname'], args['--export'], args['--extV1'], args['--extVV1'], args['--extV2'], args['--extVV2'] ]
        run(['python', 'plotXY.py'] + elems)

    elif args['<command>'] == 'dist':
        elems = [args['--tab'], args['--var'], args['--d1'], args['--d2'], args['--lat1'], args['--lat2'], args['--lon1'], args['--lon2'], args['--pname'], args['--export'], args['--extV1'], args['--extVV1'], args['--extV2'], args['--extVV2'] ]
        run(['python', 'plotDist.py'] + elems)

    elif args['<command>'] == 'section':
        elems = [args['--tab'], args['--var'], args['--d1'], args['--lat1'], args['--lat2'], args['--lon1'], args['--lon2'], args['--pname'], args['--export'], args['--extVV1'], args['--extVV2'] ]
        run(['python', 'plotSection.py'] + elems)

    elif args['<command>'] == 'depth':
        elems = [args['--tab'], args['--var'], args['--d1'], args['--lat1'], args['--lat2'], args['--lon1'], args['--lon2'], args['--pname'], args['--export'], args['--extVV1'], args['--extVV2'] ]
        run(['python', 'plotDepthProfile.py'] + elems)

    elif args['<command>'] == 'season':
        elems = [args['--tab'], args['--var'], args['--lat1'], args['--lat2'], args['--lon1'], args['--lon2'], args['--pname'], args['--export'], args['--extV1'], args['--extVV1'], args['--extV2'], args['--extVV2'] ]
        run(['python', 'plotMonthly.py'] + elems)

    elif args['<command>'] == 'cat':
        run(['python', 'getCatalog.py'])

    if args['<command>'] == 'cruise':
        if bool(int(args['--fCruise'])):
            cruiseSource = args['--cruiseTab']    # cruise table name
        else:    
            cruiseSource = args['--vcFile']       # virtaul cruise file
        elems = [args['--fCruise'], args['--cruiseTrack'], cruiseSource, args['--cruiseName'], args['--resampleRate'], args['--pname'], args['--export'], args['--sMatch'], args['--tab'], args['--var'], args['--extV1'], args['--extVV1'], args['--extV2'], args['--extVV2'] ]
        run(['python', 'plotCruise.py'] + elems)

    elif args['<command>'] in ['help', None]:
        run(['python', 'opedia.py', '--help'])

    elif args['<command>'] == 'match':
        elems = [args['--dbMatch'], args['--sourceMatch'], args['--sMatch'], args['--tMatch'], args['--tab'], args['--var'], args['--eFile'] ]
        run(['python', 'match.py'] + elems)

    elif args['<command>'] == 'advect':
        elems = [args['--dtAdvect'], args['--dirAdvect'], args['--d1'], args['--d2'], args['--latAdvect'], args['--lonAdvect'], args['--shape'], args['--match'], args['--shpFile'], args['--export'], args['--sMatch'], args['--tab'], args['--var'], args['--extV1'], args['--extVV1'], args['--extV2'], args['--extVV2'], args['--pname'] ]
        run(['python', 'Lagrangian.py'] + elems)

    elif args['<command>'] == 'eddy':
        elems = [args['--eddySource'], args['--d1'], args['--d2'], args['--lat1'], args['--lat2'], args['--lon1'], args['--lon2'], args['--shape'], args['--match'], args['--shpFile'], args['--export'], args['--sMatch'], args['--tab'], args['--var'], args['--extV1'], args['--extVV1'], args['--extV2'], args['--extVV2'], args['--pname'] ]
        run(['python', 'eddy.py'] + elems)

    elif args['<command>'] == 'ftle':
        elems = [args['--ftleSource'], args['--ftleVar'], args['--ftleValue'], args['--bkg'], args['--d1'], args['--d2'], args['--lat1'], args['--lat2'], args['--lon1'], args['--lon2'], args['--shape'], args['--match'], args['--shpFile'], args['--export'], args['--sMatch'], args['--tab'], args['--var'], args['--extV1'], args['--extVV1'], args['--extV2'], args['--extVV2'], args['--pname'] ]
        run(['python', 'ftle.py'] + elems)

    else:
        exit("%r is not an opedia command. Run 'opedia.py help'." % args['<command>'])
    


import sys
import os
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import db
import climatology as clim
import subset
from datetime import datetime, timedelta
import time
from math import pi
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.models import DatetimeTickFormatter
from bokeh.palettes import all_palettes
from bokeh.models import HoverTool
from bokeh.embed import components

def iterative(table):
    it = False
    if table.find('tblDarwin') != -1:
        it = True
    if table.find('tblPisces') != -1:
        it = True
    return it


def exportData(z, y, yErr, table, variable, date1, date2, lat1, lat2, lon1, lon2, fname):
    df = pd.DataFrame()    
    df['depth'] = z
    df[variable] = y
    df[variable+'_std'] = yErr
    if db.isClimatology(table):
        df['month1'] = clim.timeToMonth(date1)  
        df['month2'] = clim.timeToMonth(date2)  
    else:
        df['time1'] = date1
        df['time2'] = date2
    df['lat1'] = lat1
    df['lat2'] = lat2
    df['lon1'] = lon1
    df['lon2'] = lon2
    dirPath = 'data/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        
    path = dirPath + fname + '_' + table + '_' + variable + '.csv'
    df.to_csv(path, index=False)    
    return


def depthProfile_iterative(table, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag):
    if db.isClimatology(table) and db.hasField(table, 'month'):
        m1 = clim.timeToMonth(dt1)
        m2 = clim.timeToMonth(dt2)
        if m2>m1:
            timesteps = range(m1, m2+1)
        else:
            timesteps = range(m2, m1+1)
        timesteps = ['2016-%2.2d-01' % m for m in timesteps]    
    else:        
        delta = datetime.strptime(dt2, '%Y-%m-%d') - datetime.strptime(dt1, '%Y-%m-%d')
        timesteps = [(datetime.strptime(dt1, '%Y-%m-%d') + timedelta(days=x)).strftime('%Y-%m-%d') for x in range(delta.days+1)]

    zs, ys, y_stds = [], [], []
    for dt in timesteps:
        df = subset.depthProfile(table, field, dt, dt, lat1, lat2, lon1, lon2, depth1, depth2)
        if len(df[field]) < 1:
            continue
        zs.append(df['depth'])
        ys.append(df[field])
        y_stds.append(df[field + '_std'])

    depth = np.mean( np.stack(zs, axis=0), axis=0 )
    y = np.mean( np.stack(ys, axis=0), axis=0 )
    y_std = np.mean( np.stack(y_stds, axis=0), axis=0 )

    if exportDataFlag:
        exportData(depth, y, y_std, table, field, dt1, dt2, lat1, lat2, lon1, lon2, fname)    
    return depth, y, y_std


def depthProfile(table, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag):
    df = subset.depthProfile(table, field, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2)
    if exportDataFlag:
        exportData(df['depth'], df[field], df[field + '_std'], table, field, dt1, dt2, lat1, lat2, lon1, lon2, fname)    
    return df['depth'], df[field], df[field + '_std']


def plotDepthProfile(tables, variables, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag, marker='-', msize=30, clr='orangered'):
    p = []
    lw = 2
    w = 800
    h = 400
    TOOLS = 'pan,wheel_zoom,zoom_in,zoom_out,box_zoom, undo,redo,reset,tap,save,box_select,poly_select,lasso_select'
    for i in range(len(tables)):
        if not db.hasField(tables[i], 'depth'):
            continue
        if not iterative(tables[i]):    
            depths, y, yErr = depthProfile(tables[i], variables[i], dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag)
        else:    
            depths, y, yErr = depthProfile_iterative(tables[i], variables[i], dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag)
        p1 = figure(tools=TOOLS, toolbar_location="above", plot_width=w, plot_height=h)
        #p1.xaxis.axis_label = 'Depth'
        p1.yaxis.axis_label = variables[i] + ' [' + db.getVar(tables[i], variables[i]).iloc[0]['Unit'] + ']'
        leg = variables[i]
        cr = p1.circle(depths, y, fill_color="grey", hover_fill_color="firebrick", fill_alpha=0.07, hover_alpha=0.3, line_color=None, hover_line_color="white", legend=leg, size=msize)
        p1.line(depths, y, line_color=clr, line_width=lw, legend=leg)
        p1.add_tools(HoverTool(tooltips=None, renderers=[cr], mode='hline'))
        p.append(p1)
    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        
    output_file(dirPath + fname + ".html", title="Depth Profile")
    show(column(p))    
    return


def main():
    tables = sys.argv[1].split(',')      
    variables = sys.argv[2].split(',')      
    dt1 = sys.argv[3]      
    dt2 = sys.argv[4]
    lat1 = sys.argv[5]     
    lat2 = sys.argv[6]      
    lon1 = sys.argv[7]     
    lon2 = sys.argv[8]     
    depth1 = sys.argv[9]   
    depth2 = sys.argv[10]    
    fname = sys.argv[11]
    exportDataFlag = bool(int(sys.argv[12]))

    if float(lat1)>float(lat2):
        temp = lat1
        lat1 = lat2
        lat2 = temp

    if float(lon1)>float(lon2):
        temp = lon1
        lon1 = lon2
        lon2 = temp

    if float(depth1)>float(depth2):
        temp = depth1
        depth1 = depth2
        depth2 = temp

    #tic = time.clock()
    plotDepthProfile(tables, variables, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag)
    #toc = time.clock()
    #print('Fetch time: %2.2f s' % (toc-tic))


if __name__ == '__main__':
    main()   


import sys
import os
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import db
import subset
import climatology as clim
from datetime import datetime, timedelta
import time
from bokeh.plotting import figure, show, output_file
from bokeh.layouts import column
from bokeh.palettes import all_palettes
from bokeh.models import LinearColorMapper, BasicTicker, ColorBar
from bokeh.embed import components


def getBounds(varName):
    bounds = (None, None)
    if varName.find('Fe') != -1:
        bounds = (0, 1e-4)
    elif varName.find('chl') != -1:
        bounds = (0, 5e-1)
    elif varName.find('O2') != -1:
        bounds = (200, 320)
    elif varName.find('PHYC') != -1:
        bounds = (0, 4)
    elif varName.find('PP') != -1:
        bounds = (0, 4e-2)
    elif varName.find('Si') != -1:
        bounds = (10, 30)
    elif varName.find('NO3') != -1:
        bounds = (0, 20)
    elif varName.find('PO4') != -1:
        bounds = (0, 1.5)
    elif varName.find('wind_stress') != -1:
        bounds = (0, 3e-1)
    elif varName.find('sst') != -1:
        bounds = (0, 32)          
    elif varName.find('sla') != -1:
        bounds = (-0.3, 0.3)          
    return bounds

def getPalette(varName):
    paletteName = 'RdBu11'
    if varName.find('Fe') != -1:
        paletteName = 'Viridis256'
    elif varName.find('O2') != -1:
        paletteName = 'Viridis256'
    elif varName.find('PHYC') != -1:
        paletteName = 'Viridis256'
    elif varName.find('PP') != -1:
        paletteName = 'Viridis256'
    elif varName.find('Si') != -1:
        paletteName = 'Viridis256'
    elif varName.find('NO3') != -1:
        paletteName = 'Viridis256'
    elif varName.find('PO4') != -1:
        paletteName = 'Viridis256'
    elif varName.find('chl') != -1:
        paletteName = 'Viridis256'
    elif varName.find('wind_stress') != -1:
        paletteName = 'Plasma256'
    elif varName.find('sst') != -1:
        paletteName = 'Inferno256'
    elif varName.find('sla') != -1:
        paletteName = 'RdBu11'
    return paletteName


def prepareSectionQuery(table, field, date1, lat1, lat2, lon1, lon2, depth1, depth2):
    if clim.isClimatology(table, field):
        query = "SELECT [month], lat, lon, depth, %s FROM %s WHERE "
        query = query + "[month]=%d AND "
    else:
        query = "SELECT [time], lat, lon, depth, %s FROM %s WHERE "
        query = query + "[time]='%s' AND "
    query = query + "lat BETWEEN %f AND %f AND "
    query = query + "lon  BETWEEN %f AND %f AND "
    query = query + "depth BETWEEN %f AND %f "
    query = query + "ORDER BY lat, lon, depth "

    if clim.isClimatology(table, field):
        month = clim.timeToMonth(date1)
        query = query % (field, table, month, float(lat1), float(lat2), float(lon1), float(lon2), float(depth1), float(depth2))
    else:
        query = query % (field, table, date1, float(lat1), float(lat2), float(lon1), float(lon2), float(depth1), float(depth2))
    return query


def exportData(df, path):
    df.to_csv(path, index=False)    
    return


def sectionMap(tables, variabels, dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag):
    data, lats, lons, subs, frameVars = [], [], [], [], []
    for i in range(len(tables)):
        if not db.hasField(tables[i], 'depth'):
            continue        
        #query = prepareSectionQuery(tables[i], variabels[i], dt, lat1, lat2, lon1, lon2, depth1, depth2)
        #df = db.dbFetch(query)

        df = subset.section(tables[i], variabels[i], dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2)
        if len(df) < 1:
            continue

        ############### export retrieved data ###############
        if exportDataFlag:      # export data
            dirPath = 'data/'
            if not os.path.exists(dirPath):
                os.makedirs(dirPath)                
            exportData(df, path=dirPath + fname + '_' + tables[i] + '_' + variabels[i] + '.csv')
        #####################################################

        times = df[df.columns[0]].unique() 
        lats = df.lat.unique()
        lons = df.lon.unique()
        depths = df.depth.unique()
        shape = (len(lats), len(lons), len(depths))

        hours =  [None]
        if 'hour' in df.columns:
            hours = df.hour.unique()

        unit =  ' [' + db.getVar(tables[i], variabels[i]).iloc[0]['Unit'] + ']'

        for t in times:
            for h in hours:
                frame = df[df[df.columns[0]] == t]
                sub = variabels[i] + unit + ', ' + df.columns[0] + ': ' + str(t) 
                if h != None:
                    frame = frame[frame['hour'] == h]
                    sub = sub + ', hour: ' + str(h) + 'hr'
                try:    
                    shot = frame[variabels[i]].values.reshape(shape)
                except Exception as e:
                    continue    
                data.append(shot)
                frameVars.append(variabels[i])
                subs.append(sub)

    bokehSec(data=data, subject=subs, fname=fname, lat=lats, lon=lons, depth=depths, variabels=frameVars)
    return


def bokehSec(data, subject, fname, lat, lon, depth, variabels):
    TOOLS="hover,crosshair,pan,wheel_zoom,zoom_in,zoom_out,box_zoom,undo,redo,reset,tap,save,box_select,poly_select,lasso_select,"
    w = 1000
    h = 500
    p = []
    data_org = list(data)
    for ind in range(len(data_org)):
        data = data_org[ind]
        bounds = getBounds(variabels[ind])
        paletteName = getPalette(variabels[ind])
        if bounds[0]==None:
            color_mapper = LinearColorMapper(palette=paletteName)
        else:   
            color_mapper = LinearColorMapper(palette=paletteName, low=bounds[0], high=bounds[1])     


        if len(lon) > len(lat):
            p1 = figure(tools=TOOLS, toolbar_location="above", title=subject[ind], plot_width=w, plot_height=h, x_range=(np.min(lon), np.max(lon)), y_range=(np.min(depth), np.max(depth)))
            data = np.nanmean(data, axis=0)
            data = np.transpose(data)
            data = np.squeeze(data)
            p1.xaxis.axis_label = 'Longitude'
            p1.image(image=[data], color_mapper=color_mapper, x=np.min(lon), y=np.max(depth), dw=np.max(lon)-np.min(lon), dh=np.max(depth)-np.min(depth))
        else:
            p1 = figure(tools=TOOLS, toolbar_location="above", title=subject[ind], plot_width=w, plot_height=h, x_range=(np.min(lat), np.max(lat)), y_range=(np.min(depth), np.max(depth)))
            data = np.nanmean(data, axis=1)
            data = np.transpose(data)
            data = np.squeeze(data)
            p1.xaxis.axis_label = 'Latitude'
            '''
            if ind==0:
                data1 = data
            if ind==1:
                data = data / data1
            '''
            p1.image(image=[data], color_mapper=color_mapper, x=np.min(lat), y=np.min(depth), dw=np.max(lat)-np.min(lat), dh=np.max(depth)-np.min(depth))

        p1.yaxis.axis_label = 'depth [m]'
        color_bar = ColorBar(color_mapper=color_mapper, ticker=BasicTicker(),
                        label_standoff=12, border_line_color=None, location=(0,0))
        p1.add_layout(color_bar, 'right')
        p.append(p1)
    dirPath = 'embed/'
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)        
    output_file(dirPath + fname + ".html", title="Regional Map")
    show(column(p))
    return


def main():
    tables = sys.argv[1]      
    variables = sys.argv[2]      
    dt1 = sys.argv[3]      
    dt2 = sys.argv[4]      
    lat1 = sys.argv[5]      
    lat2 = sys.argv[6]      
    lon1 = sys.argv[7]      
    lon2 = sys.argv[8]      
    depth1 = sys.argv[9]      
    depth2 = sys.argv[10]      
    fname = sys.argv[11]
    exportDataFlag = bool(int(sys.argv[12]))

    sectionMap(tables.split(','), variables.split(','), dt1, dt2, lat1, lat2, lon1, lon2, depth1, depth2, fname, exportDataFlag)


if __name__ == '__main__':
    main()    